/*
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.vm.compiler.agent.internal;

import java.io.IOException;
import java.net.URI;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.felix.scr.annotations.Activate;
import org.apache.felix.scr.annotations.Component;
import org.apache.felix.scr.annotations.ConfigurationPolicy;
import org.apache.felix.scr.annotations.Modified;
import org.apache.felix.scr.annotations.Reference;
import org.apache.felix.scr.annotations.Service;

import com.redhat.thermostat.agent.http.HttpRequestService;
import com.redhat.thermostat.common.config.experimental.ConfigurationInfoSource;
import com.redhat.thermostat.common.plugin.BackendConfigurationUtil;
import com.redhat.thermostat.common.plugin.PluginDAOBase;
import com.redhat.thermostat.common.plugin.SystemID;
import com.redhat.thermostat.common.utils.LoggingUtils;
import com.redhat.thermostat.vm.compiler.model.VmCompilerStat;
import com.redhat.thermostat.vm.compiler.model.VmCompilerStatTypeAdapter;

@Component(
        policy = ConfigurationPolicy.REQUIRE,
        configurationPid = "vm-compiler"
)
@Service(value = VmCompilerStatDao.class)
public class VmCompilerStatDaoImpl extends PluginDAOBase<VmCompilerStat> implements VmCompilerStatDao {

    private static final Logger logger = LoggingUtils.getLogger(VmCompilerStatDaoImpl.class);
    private static final String PLUGIN_ID = "vm-compiler";

    private final JsonHelper jsonHelper;
    private final ConfigurationCreator configCreator;
    private URI gatewayURL;
    private BackendConfigurationUtil config;

    @Reference
    private ConfigurationInfoSource configurationInfoSource;

    @Reference
    private HttpRequestService httpRequestService;

    @Reference
    private SystemID systemID;

    public VmCompilerStatDaoImpl() {
        this(new JsonHelper(new VmCompilerStatTypeAdapter()), new ConfigurationCreator());
    }

    VmCompilerStatDaoImpl(JsonHelper jsonHelper, ConfigurationCreator creator) {
        this.jsonHelper = jsonHelper;
        this.configCreator = creator;
    }

    @Activate
    public void activate() throws Exception {
        config = configCreator.create(configurationInfoSource);
        this.gatewayURL = config.getGatewayURL();
    }


    /*
     * Default OSGI behaviour is to restart the component when configuration changes.
     * The @modified method acts as a callback to prevent this from happening since
     * we don't need to restart the component for our use-case.
     */
    @Modified
    protected void modified() {
        logger.config("Doing nothing on configuration change for plugin vm-compiler");
    }

    @Override
    protected String toJsonString(VmCompilerStat obj) throws IOException {
        return jsonHelper.toJson(Arrays.asList(obj));
    }

    @Override
    protected HttpRequestService getHttpRequestService() {
        return httpRequestService;
    }

    @Override
    protected BackendConfigurationUtil getConfig() {
        return config;
    }

    @Override
    protected URI getPostURI(URI basepath, VmCompilerStat obj) {
        return basepath.resolve("systems/" + systemID.getSystemID() + "/jvms/" + obj.getJvmId());
    }

    @Override
    protected Logger getLogger() {
        return logger;
    }

    public String getPluginId() {
        return PLUGIN_ID;
    }

    protected void bindSystemID(final SystemID id) {
        this.systemID = id;
    }

    protected void bindConfigurationInfoSource(ConfigurationInfoSource cfg) {
        this.configurationInfoSource = cfg;
    }

    protected void bindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = httpRequestService;
    }

    protected void unbindHttpRequestService(HttpRequestService httpRequestService) {
        this.httpRequestService = null;
        logger.log(Level.INFO, "Unbound HTTP service. Further attempts to store data will fail until bound again.");
    }

    // For testing purposes
    static class JsonHelper {
        private final VmCompilerStatTypeAdapter typeAdapter;

        public JsonHelper(VmCompilerStatTypeAdapter typeAdapter) {
            this.typeAdapter = typeAdapter;
        }

        public String toJson(List<VmCompilerStat> info) throws IOException {
            return typeAdapter.toJson(info);
        }
    }

    // For Testing purposes
    static class ConfigurationCreator {
        BackendConfigurationUtil create(ConfigurationInfoSource source) {
            return new BackendConfigurationUtil(source, PLUGIN_ID);
        }
    }
}
